@@ -1,27 +1,7 @@
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 
-// ref.) https://en.wikipedia.org/wiki/Memory_ordering
-fn memory_barrier(_arg: &Vec<&str>) {
-    #[cfg(target_feature = "sse2")]
-    {
-        #[cfg(target_arch = "x86")]
-        use std::arch::x86 as mmx;
-        #[cfg(target_arch = "x86_64")]
-        use std::arch::x86_64 as mmx;
-
-        unsafe { mmx::_mm_mfence() };
-    }
-    /*
-    #[cfg(target_arch = "arm")]
-    {
-        unsafe { core::arch::arm::__dmb(_arg) };
-    }
-    #[cfg(target_arch = "aarch64")]
-    {
-        unsafe { core::arch::aarch64::__dmb(_arg) };
-    }
-    */
-}
+mod barrier;
+use barrier::memory_barrier;
 
 #[inline(never)]
 fn process_std_memchr(texts: &[&str], pat_byte: u8) -> usize {
@@ -39,13 +19,62 @@ fn process_std_memchr(texts: &[&str], pat_byte: u8) -> usize {
                 break;
             }
         }
-        /*
-        for i in 0..line_len {
-            if line_bytes[i] == pat_byte {
+    }
+    found
+}
+
+#[inline(never)]
+fn process_libc_memchr(texts: &[&str], pat_byte: u8) -> usize {
+    // original libc function
+    extern {
+        fn memchr(cx: *const u8, c: i32, n: usize) -> *const u8;
+    }
+    #[inline(always)]
+    fn _x_libc_memchr(buf: &[u8], c: u8) -> Option<usize> {
+        let cx = buf.as_ptr();
+        let len = buf.len();
+        let r = unsafe { memchr(cx, c.into(), len) };
+        if !r.is_null() {
+            Some(r as usize - cx as usize)
+        } else {
+            None
+        }
+    }
+    //
+    let mut found: usize = 0;
+    for line in texts {
+        let line_bytes = line.as_bytes();
+        let line_len = line_bytes.len();
+        let mut curr_idx = 0;
+        while curr_idx < line_len {
+            let r = _x_libc_memchr(&line_bytes[curr_idx..], pat_byte);
+            if let Some(pos) = r {
                 found += 1;
+                curr_idx = pos + curr_idx + 1;
+            } else {
+                break;
+            }
+        }
+    }
+    found
+}
+
+#[inline(never)]
+fn process_memchr_memchr(texts: &[&str], pat_byte: u8) -> usize {
+    let mut found: usize = 0;
+    for line in texts {
+        let line_bytes = line.as_bytes();
+        let line_len = line_bytes.len();
+        let mut curr_idx = 0;
+        while curr_idx < line_len {
+            let r = memchr::memchr(pat_byte, &line_bytes[curr_idx..]);
+            if let Some(pos) = r {
+                found += 1;
+                curr_idx = pos + curr_idx + 1;
+            } else {
+                break;
             }
         }
-        */
     }
     found
 }
@@ -90,6 +119,28 @@ fn process_memx_memchr_basic(texts: &[&str], pat_byte: u8) -> usize {
     found
 }
 
+#[cfg(all(any(target_arch = "x86_64", target_arch = "x86"), target_feature = "sse2"))]
+#[inline(never)]
+fn process_memx_memchr_sse2(texts: &[&str], pat_byte: u8) -> usize {
+    let mut found: usize = 0;
+    for line in texts {
+        let line_bytes = line.as_bytes();
+        let line_len = line_bytes.len();
+        let mut curr_idx = 0;
+        while curr_idx < line_len {
+            let r = unsafe { memx::arch::x86::_memchr_sse2(&line_bytes[curr_idx..], pat_byte) };
+            if let Some(pos) = r {
+                found += 1;
+                curr_idx = pos + curr_idx + 1;
+            } else {
+                break;
+            }
+        }
+    }
+    found
+}
+
+/*
 #[inline(never)]
 fn process_memx_memchr_libc(texts: &[&str], pat_byte: u8) -> usize {
     let mut found: usize = 0;
@@ -109,26 +160,7 @@ fn process_memx_memchr_libc(texts: &[&str], pat_byte: u8) -> usize {
     }
     found
 }
-
-#[inline(never)]
-fn process_memchr_memchr(texts: &[&str], pat_byte: u8) -> usize {
-    let mut found: usize = 0;
-    for line in texts {
-        let line_bytes = line.as_bytes();
-        let line_len = line_bytes.len();
-        let mut curr_idx = 0;
-        while curr_idx < line_len {
-            let r = memchr::memchr(pat_byte, &line_bytes[curr_idx..]);
-            if let Some(pos) = r {
-                found += 1;
-                curr_idx = pos + curr_idx + 1;
-            } else {
-                break;
-            }
-        }
-    }
-    found
-}
+*/
 
 mod create_data;
 
@@ -138,14 +170,21 @@ fn criterion_benchmark(c: &mut Criterion) {
     //
     let n = process_std_memchr(black_box(&vv), black_box(pat_byte));
     assert_eq!(n, match_cnt);
+    let n = process_libc_memchr(black_box(&vv), black_box(pat_byte));
+    assert_eq!(n, match_cnt);
+    let n = process_memchr_memchr(black_box(&vv), black_box(pat_byte));
+    assert_eq!(n, match_cnt);
     let n = process_memx_memchr(black_box(&vv), black_box(pat_byte));
     assert_eq!(n, match_cnt);
     let n = process_memx_memchr_basic(black_box(&vv), black_box(pat_byte));
     assert_eq!(n, match_cnt);
-    let n = process_memx_memchr_libc(black_box(&vv), black_box(pat_byte));
-    assert_eq!(n, match_cnt);
-    let n = process_memchr_memchr(black_box(&vv), black_box(pat_byte));
-    assert_eq!(n, match_cnt);
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "x86"), target_feature = "sse2"))]
+    {
+        let n = process_memx_memchr_sse2(black_box(&vv), black_box(pat_byte));
+        assert_eq!(n, match_cnt);
+    }
+    //let n = process_memx_memchr_libc(black_box(&vv), black_box(pat_byte));
+    //assert_eq!(n, match_cnt);
     memory_barrier(&vv);
     //
     c.bench_function("std_memchr", |b| {
@@ -154,6 +193,18 @@ fn criterion_benchmark(c: &mut Criterion) {
             memory_barrier(&vv);
         })
     });
+    c.bench_function("libc_memchr", |b| {
+        b.iter(|| {
+            let _r = process_libc_memchr(black_box(&vv), black_box(pat_byte));
+            memory_barrier(&vv);
+        })
+    });
+    c.bench_function("memchr_memchr", |b| {
+        b.iter(|| {
+            let _r = process_memchr_memchr(black_box(&vv), black_box(pat_byte));
+            memory_barrier(&vv);
+        })
+    });
     c.bench_function("memx_memchr", |b| {
         b.iter(|| {
             let _r = process_memx_memchr(black_box(&vv), black_box(pat_byte));
@@ -166,18 +217,21 @@ fn criterion_benchmark(c: &mut Criterion) {
             memory_barrier(&vv);
         })
     });
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "x86"), target_feature = "sse2"))]
+    c.bench_function("memx_memchr_sse2", |b| {
+        b.iter(|| {
+            let _r = process_memx_memchr_sse2(black_box(&vv), black_box(pat_byte));
+            memory_barrier(&vv);
+        })
+    });
+    /*
     c.bench_function("memx_memchr_libc", |b| {
         b.iter(|| {
             let _r = process_memx_memchr_libc(black_box(&vv), black_box(pat_byte));
             memory_barrier(&vv);
         })
     });
-    c.bench_function("memchr_memchr", |b| {
-        b.iter(|| {
-            let _r = process_memchr_memchr(black_box(&vv), black_box(pat_byte));
-            memory_barrier(&vv);
-        })
-    });
+    */
 }
 
 criterion_group! {
