@@ -1,27 +1,7 @@
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 
-// ref.) https://en.wikipedia.org/wiki/Memory_ordering
-fn memory_barrier(_arg: &mut [Vec<u8>]) {
-    #[cfg(target_feature = "sse2")]
-    {
-        #[cfg(target_arch = "x86")]
-        use std::arch::x86 as mmx;
-        #[cfg(target_arch = "x86_64")]
-        use std::arch::x86_64 as mmx;
-
-        unsafe { mmx::_mm_mfence() };
-    }
-    /*
-    #[cfg(target_arch = "arm")]
-    {
-        unsafe { core::arch::arm::__dmb(_arg) };
-    }
-    #[cfg(target_arch = "aarch64")]
-    {
-        unsafe { core::arch::aarch64::__dmb(_arg) };
-    }
-    */
-}
+mod barrier;
+use barrier::memory_barrier;
 
 #[inline(never)]
 fn process_std_memset(texts: &mut [Vec<u8>], pat_u8: u8) {
@@ -34,6 +14,29 @@ fn process_std_memset(texts: &mut [Vec<u8>], pat_u8: u8) {
     }
 }
 
+#[inline(never)]
+fn process_libc_memset(texts: &mut [Vec<u8>], pat_u8: u8) {
+    // original libc function
+    extern {
+        fn memset(dest: *mut u8, c: i32, n: usize) -> *mut u8;
+    }
+    #[inline(always)]
+    fn _x_libc_memset(dest: &mut [u8], c: u8, len: usize) -> bool {
+        let dest_ptr = dest.as_mut_ptr();
+        if dest.len() < len {
+            return false;
+        }
+        let _r = unsafe { memset(dest_ptr, c.into(), len) };
+        true
+    }
+    //
+    for i in 0..texts.len() {
+        let line_bytes = &mut texts[i];
+        let line_len = line_bytes.len();
+        let _ = _x_libc_memset(&mut *line_bytes, pat_u8, line_len);
+    }
+}
+
 #[inline(never)]
 fn process_memx_memset(texts: &mut [Vec<u8>], pat_u8: u8) {
     for i in 0..texts.len() {
@@ -52,6 +55,17 @@ fn process_memx_memset_basic(texts: &mut [Vec<u8>], pat_u8: u8) {
     }
 }
 
+#[cfg(all(any(target_arch = "x86_64", target_arch = "x86"), target_feature = "sse2"))]
+#[inline(never)]
+fn process_memx_memset_sse2(texts: &mut [Vec<u8>], pat_u8: u8) {
+    for i in 0..texts.len() {
+        let line_bytes = &mut texts[i];
+        let line_len = line_bytes.len();
+        let _ = unsafe { memx::arch::x86::_memset_sse2(&mut *line_bytes, pat_u8, line_len) };
+    }
+}
+
+/*
 #[inline(never)]
 fn process_memx_memset_libc(texts: &mut [Vec<u8>], pat_u8: u8) {
     for i in 0..texts.len() {
@@ -60,6 +74,7 @@ fn process_memx_memset_libc(texts: &mut [Vec<u8>], pat_u8: u8) {
         let _ = memx::libc::memset_libc(&mut *line_bytes, pat_u8, line_len);
     }
 }
+*/
 
 mod create_data;
 
@@ -68,9 +83,12 @@ fn criterion_benchmark(c: &mut Criterion) {
     let pat_u8 = b'A';
     //
     process_std_memset(black_box(&mut v), black_box(pat_u8));
+    process_libc_memset(black_box(&mut v), black_box(pat_u8));
     process_memx_memset(black_box(&mut v), black_box(pat_u8));
     process_memx_memset_basic(black_box(&mut v), black_box(pat_u8));
-    process_memx_memset_libc(black_box(&mut v), black_box(pat_u8));
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "x86"), target_feature = "sse2"))]
+    process_memx_memset_sse2(black_box(&mut v), black_box(pat_u8));
+    //process_memx_memset_libc(black_box(&mut v), black_box(pat_u8));
     //
     c.bench_function("std_memset", |b| {
         b.iter(|| {
@@ -78,6 +96,12 @@ fn criterion_benchmark(c: &mut Criterion) {
             memory_barrier(&mut v);
         })
     });
+    c.bench_function("libc_memset", |b| {
+        b.iter(|| {
+            process_libc_memset(black_box(&mut v), black_box(pat_u8));
+            memory_barrier(&mut v);
+        })
+    });
     c.bench_function("memx_memset", |b| {
         b.iter(|| {
             process_memx_memset(black_box(&mut v), black_box(pat_u8));
@@ -90,12 +114,21 @@ fn criterion_benchmark(c: &mut Criterion) {
             memory_barrier(&mut v);
         })
     });
+    #[cfg(all(any(target_arch = "x86_64", target_arch = "x86"), target_feature = "sse2"))]
+    c.bench_function("memx_memset_sse2", |b| {
+        b.iter(|| {
+            process_memx_memset_sse2(black_box(&mut v), black_box(pat_u8));
+            memory_barrier(&mut v);
+        })
+    });
+    /*
     c.bench_function("memx_memset_libc", |b| {
         b.iter(|| {
             process_memx_memset_libc(black_box(&mut v), black_box(pat_u8));
             memory_barrier(&mut v);
         })
     });
+    */
 }
 
 criterion_group! {
