@@ -1,5 +1,5 @@
-use crate::plus_offset_from;
 use crate::mem as basic;
+use crate::plus_offset_from;
 
 #[cfg(target_arch = "x86")]
 use core::arch::x86 as mmx;
@@ -75,6 +75,9 @@ macro_rules! _unroll_one_chr_16_aa {
 
 #[inline(always)]
 fn _memchr_sse2_impl(buf: &[u8], c: u8) -> Option<usize> {
+    if buf.len() == 0 {
+        return None;
+    }
     //
     let buf_len = buf.len();
     let mut buf_ptr = buf.as_ptr();
