@@ -52,6 +52,9 @@ fn _memcpy_sse2_impl(dst: &mut [u8], src: &[u8]) -> Result<(), RangeError> {
     if dst_len < src_len {
         return Err(RangeError);
     }
+    if src_len == 0 {
+        return Ok(());
+    }
     let mut a_ptr = dst.as_mut_ptr();
     let mut b_ptr = src.as_ptr();
     let end_ptr = unsafe { b_ptr.add(src_len) };
@@ -121,7 +124,6 @@ fn _cpy_small(
     (a_ptr, b_ptr)
 }
 
-
 #[inline(always)]
 fn _cpy_medium(
     dst_ptr: *mut u8,
@@ -291,7 +293,6 @@ fn _cpy_unroll_16x8(a_ptr: *mut u8, b_ptr: *const u8, loop_size: usize) {
     unsafe { _mm_store_si128(aa7_ptr, mm_b7) };
 }
 
-
 #[inline(always)]
 fn _cpy_unroll_16x6(a_ptr: *mut u8, b_ptr: *const u8, loop_size: usize) {
     let aa0_ptr = unsafe { a_ptr.add(loop_size * 0) } as *mut __m128i;
